'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _dynamoDb = require('./dynamo-db');

var _dynamoDb2 = _interopRequireDefault(_dynamoDb);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class DynamoTable {
  constructor({
    tableName = '',
    schema = {},
    primaryKey = ''
  }) {
    this.DynamoDB = _dynamoDb2.default;
    this.tableName = tableName;
    this.schema = schema;
    this.primaryKey = primaryKey;
  }

  setTableName(tableName) {
    if (!tableName) {
      throw new Error('tableName needs to be set');
    }
    this.tableName = tableName;

    return this;
  }

  setSchema(schema) {
    if (!schema) {
      throw new Error('schema needs to be set');
    }
    this.schema = schema;

    return this;
  }

  setPrimaryKey(primaryKey) {
    if (!primaryKey) {
      throw new Error('primaryKey needs to be set');
    }
    this.primaryKey = primaryKey;

    return this;
  }

  add(item, options = {}) {
    this.validateInstance();

    return new Promise((resolve, reject) => _dynamoDb2.default.putItem(_extends({
      TableName: this.tableName,
      ReturnValues: 'NONE',
      Item: (0, _helpers.buildDBItem)(item, this.schema)
    }, options), (err, res) => {
      if (err) {
        reject(err);
      } else {
        resolve(res);
      }
    }));
  }

  delete(itemId, options = {}) {
    this.validateInstance();

    return new Promise((resolve, reject) => _dynamoDb2.default.deleteItem(_extends({
      TableName: this.tableName,
      Key: this.buildPrimaryKey(itemId)
    }, options), (err, res) => {
      if (err) {
        reject(err);
      } else {
        resolve(res);
      }
    }));
  }

  get(itemId, options = {}) {
    this.validateInstance();

    return new Promise((resolve, reject) => _dynamoDb2.default.getItem(_extends({
      TableName: this.tableName,
      Key: this.buildPrimaryKey(itemId)
    }, options), (err, res) => {
      if (err) {
        reject(err);
      } else {
        resolve((0, _helpers.parseDBItem)(res.Item || {}, this.schema));
      }
    }));
  }

  update(itemId, item, options = {}) {
    this.validateInstance();

    const Item = (0, _helpers.buildDBItem)(item, this.schema);
    const keys = Object.keys(item);
    let keyCounter = 96;

    const keyMap = keys.reduce((acc, key) => {
      keyCounter++;

      return _extends({}, acc, {
        [key]: String.fromCharCode(keyCounter).toLowerCase()
      });
    }, {});

    const updateExpression = keys.reduce((acc, key) => {
      acc.push(`#${keyMap[key].toUpperCase()} = :${keyMap[key]}`);

      return acc;
    }, []).join(', ');

    return new Promise((resolve, reject) => _dynamoDb2.default.updateItem(_extends({
      TableName: this.tableName,
      Key: this.buildPrimaryKey(itemId),
      ExpressionAttributeNames: keys.reduce((acc, key) => _extends({}, acc, {
        [`#${keyMap[key].toUpperCase()}`]: key
      }), {}),
      ExpressionAttributeValues: keys.reduce((acc, key) => _extends({}, acc, {
        [`:${keyMap[key]}`]: Item[key]
      }), {}),
      ReturnValues: 'ALL_NEW',
      UpdateExpression: `SET ${updateExpression}`
    }, options), (err, res) => {
      if (err) {
        reject(err);
      } else {
        resolve(res);
      }
    }));
  }

  validateInstance() {
    if (!this.tableName || !this.schema || !this.primaryKey) {
      throw new Error('require tableName, schema, and primaryKey to make requests to DynamoDB');
    } else if (!this.schema[this.primaryKey]) {
      throw new Error('Primary key needs to exist within schema');
    } else {
      return true;
    }
  }

  buildPrimaryKey(itemId) {
    return {
      [this.primaryKey]: { [this.schema[this.primaryKey]]: itemId }
    };
  }
}
exports.default = DynamoTable;